-- Copyright (C) 2012 Nicholas Carlson
--
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the "Software"),
-- to deal in the Software without restriction, including without limitation
-- the rights to use, copy, modify, merge, publish, distribute, sublicense,
-- and/or sell copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-- DEALINGS IN THE SOFTWARE.

local sprite = {}
sprite.__index = sprite

local function ctor(path, center_x, center_y)
  local v = {
    image = nil;
    geometry = nil;
    angle = 0;
    x = gWindow.width / 2;
    y = gWindow.height / 2;
    scale_x = 1.0;
    scale_y = 1.0;
  }
  setmetatable(v, sprite)

  if path and center_x and center_y then
    v:loadImage(path)
    v:createGeometry(center_x, center_y)
  end

  return v
end

function sprite:draw(_x, _y, _angle, _scale_x, _scale_y)
  local x = _x or self.x or 0
  local y = _y or self.y or 0
  local angle = _angle or self.angle or 0
  local scale_x = _scale_x or self.scale_x or 1
  local scale_y = _scale_y or self.scale_y or 1
  love.graphics.draw(self.geometry, x, y, angle, scale_x, scale_y)  
end


function sprite:loadImage(path)
  self.image = love.graphics.newImage(path)
end


function sprite:createGeometry(center_x, center_y)
  local width = self.image:getWidth()
  local height = self.image:getHeight()
  local x1 = -center_x
  local x2 = x1 + width
  local x3 = x2
  local x4 = x1
  local y1 = -center_y
  local y2 = y1
  local y3 = y1 + height
  local y4 = y3
  local vertices = {
    { x1, y1, 0, 0 };
    { x2, y2, 1, 0 };
    { x3, y3, 1, 1 };
    { x4, y4, 0, 1 };
  }
  local vertexmap = { 1, 2, 3, 4 }
  print(type(self.image))
  self.geometry = love.graphics.newMesh(vertices, self.image, "fan")  
end

return setmetatable(
  {
    ctor = ctor;
  },
  {__call = function(_,...) return ctor(...) end})